# Import libnacl libs
import libnacl.secret_easy
# Import python libs
import unittest

class TestSecretEasy(unittest.TestCase):
    '''
    '''
    def test_secret(self):
        msg = b'But then of course African swallows are not migratory.'
        box = libnacl.secret_easy.SecretBoxEasy()
        ctxt = box.encrypt(msg)
        self.assertNotEqual(msg, ctxt)
        box2 = libnacl.secret_easy.SecretBoxEasy(box.sk)
        clear1 = box.decrypt(ctxt)
        self.assertEqual(msg, clear1)
        clear2 = box2.decrypt(ctxt)
        self.assertEqual(clear1, clear2)
        ctxt2 = box2.encrypt(msg)
        clear3 = box.decrypt(ctxt2)
        self.assertEqual(clear3, msg)




"""
Basic tests for randombytes_* functions
"""

import libnacl
import unittest


class TestRandomBytes(unittest.TestCase):
    def test_randombytes_random(self):
        self.assertIsInstance(libnacl.randombytes_random(), int)

    def test_randombytes_uniform(self):
        self.assertIsInstance(libnacl.randombytes_uniform(200), int)

        freq = {libnacl.randombytes_uniform(256): 1 for _ in range(65536)}

        self.assertEqual(256, len(freq))
        self.assertTrue(all(freq.values()))

    def test_randombytes(self):
        'copied from libsodium default/randombytes.c'

        data = libnacl.randombytes(65536)

        freq = {x: 1 for x in data}

        self.assertEqual(256, len(freq))
        self.assertTrue(all(freq.values()))

    def test_randombytes_buf_deterministic(self):

        seed = libnacl.randombytes_buf(32)
        seed2 = libnacl.randombytes_buf(32)
        data = libnacl.randombytes_buf_deterministic(32, seed)
        data2 = libnacl.randombytes_buf_deterministic(32, seed)
        data3 = libnacl.randombytes_buf_deterministic(32, seed2)

        self.assertEqual(32, len(data))
        self.assertEqual(32, len(data))
        self.assertEqual(32, len(data))
        self.assertEqual(data, data2)
        self.assertNotEqual(data, data3)

    def test_crypto_kdf_keygen(self):

        master_key = libnacl.crypto_kdf_keygen()

        freq = {x: 1 for x in master_key}

        self.assertEqual(32, len(master_key))
        self.assertTrue(all(freq.values()))

    def test_crypto_kdf_derive_from_key(self):

      master_key = libnacl.crypto_kdf_keygen()
      subkey = libnacl.crypto_kdf_derive_from_key(16, 1, "Examples", master_key)
      subkey2 = libnacl.crypto_kdf_derive_from_key(16, 1, "Examples", master_key)
      subkey3 = libnacl.crypto_kdf_derive_from_key(16, 2, "Examples", master_key)

      self.assertEqual(16, len(subkey))
      self.assertEqual(16, len(subkey2))
      self.assertEqual(16, len(subkey3))
      self.assertEqual(subkey, subkey2)
      self.assertNotEqual(subkey, subkey3)
      
    def test_crypto_kx_keypair(self):
      pk, sk = libnacl.crypto_kx_keypair()
      self.assertEqual(32, len(pk))
      self.assertEqual(32, len(sk))

    def test_crypto_kx_seed_keypair(self):
      seed = libnacl.randombytes_buf(32)
      seed2 = libnacl.randombytes_buf(32)
      pk, sk = libnacl.crypto_kx_seed_keypair(seed)
      pk2, sk2 = libnacl.crypto_kx_seed_keypair(seed)
      pk3, sk3 = libnacl.crypto_kx_seed_keypair(seed2)

      self.assertEqual(pk, pk2)
      self.assertNotEqual(pk, pk3)
      self.assertEqual(sk, sk2)
      self.assertNotEqual(sk, sk3)

    def test_crypto_kx_client_session_keys(self):
      client_pk, client_sk = libnacl.crypto_kx_keypair()
      server_pk, server_sk = libnacl.crypto_kx_keypair()
      rx, tx, status = libnacl.crypto_kx_client_session_keys(client_pk, client_sk, server_pk)
      rx2, tx2, status = libnacl.crypto_kx_client_session_keys(client_pk, client_sk, server_pk)
  
      self.assertEqual(32, len(rx))
      self.assertEqual(32, len(tx))
      self.assertEqual(rx, rx2)
      self.assertEqual(tx, tx2)

    def test_crypto_kx_server_session_keys(self):
      client_pk, client_sk = libnacl.crypto_kx_keypair()
      server_pk, server_sk = libnacl.crypto_kx_keypair()
      rx, tx, status = libnacl.crypto_kx_server_session_keys(client_pk, client_sk, server_pk)
      rx2, tx2, status = libnacl.crypto_kx_server_session_keys(client_pk, client_sk, server_pk)
      
      self.assertEqual(32, len(rx))
      self.assertEqual(32, len(tx))
      self.assertEqual(rx, rx2)
      self.assertEqual(tx, tx2)



# -*- coding: utf-8 -*-
'''
High level routines to maintain signing keys and to sign and verify messages
'''
# Import libancl libs
import libnacl
import libnacl.base
import libnacl.encode


class Signer(libnacl.base.BaseKey):
    '''
    The tools needed to sign messages
    '''
    def __init__(self, seed=None):
        '''
        Create a signing key, if not seed it supplied a keypair is generated
        '''
        if seed:
            if len(seed) != libnacl.crypto_sign_SEEDBYTES:
                raise ValueError('Invalid seed bytes')
            self.vk, self.sk = libnacl.crypto_sign_seed_keypair(seed)
        else:
            seed = libnacl.randombytes(libnacl.crypto_sign_SEEDBYTES)
            self.vk, self.sk = libnacl.crypto_sign_seed_keypair(seed)
        self.seed = seed

    def sign(self, msg):
        '''
        Sign the given message with this key
        '''
        return libnacl.crypto_sign(msg, self.sk)

    def signature(self, msg):
        '''
        Return just the signature for the message
        '''
        return libnacl.crypto_sign(msg, self.sk)[:libnacl.crypto_sign_BYTES]


class Verifier(libnacl.base.BaseKey):
    '''
    Verify signed messages
    '''
    def __init__(self, vk_hex):
        '''
        Create a verification key from a hex encoded vkey
        '''
        self.vk = libnacl.encode.hex_decode(vk_hex)

    def verify(self, msg):
        '''
        Verify the message with tis key
        '''
        return libnacl.crypto_sign_open(msg, self.vk)



# Import libnacl libs
import libnacl
import libnacl.utils

# Import python libs
import unittest


class TestSecret(unittest.TestCase):
    """
    Test secret functions
    """
    def test_secretbox(self):
        msg = b'Are you suggesting coconuts migrate?'

        nonce = libnacl.utils.rand_nonce()
        key = libnacl.utils.salsa_key()

        c = libnacl.crypto_secretbox(msg, nonce, key)
        m = libnacl.crypto_secretbox_open(c, nonce, key)
        self.assertEqual(msg, m)

        with self.assertRaises(ValueError):
            libnacl.crypto_secretbox(msg, b'too_short', key)

        with self.assertRaises(ValueError):
            libnacl.crypto_secretbox(msg, nonce, b'too_short')

        with self.assertRaises(ValueError):
            libnacl.crypto_secretbox_open(c, b'too_short', key)

        with self.assertRaises(ValueError):
            libnacl.crypto_secretbox_open(c, nonce, b'too_short')



# -*- coding: utf-8 -*-
'''
Utilities to make secret box easy encryption simple
'''
# Import libnacl
import libnacl
import libnacl.utils
import libnacl.base


class SecretBoxEasy(libnacl.base.BaseKey):
    '''
    Manage symetric encryption using the salsa20 algorithm
    '''
    def __init__(self, key=None):
        if key is None:
            key = libnacl.utils.salsa_key()
        if len(key) != libnacl.crypto_secretbox_KEYBYTES:
            raise ValueError('Invalid key')
        self.sk = key

    def encrypt(self, msg, nonce=None, pack_nonce=True):
        '''
        Encrypt the given message. If a nonce is not given it will be
        generated via the rand_nonce function
        '''
        if nonce is None:
            nonce = libnacl.utils.rand_nonce()
        if len(nonce) != libnacl.crypto_secretbox_NONCEBYTES:
            raise ValueError('Invalid nonce size')
        ctxt = libnacl.crypto_secretbox_easy(msg, nonce, self.sk)
        if pack_nonce:
            return nonce + ctxt
        else:
            return nonce, ctxt

    def decrypt(self, ctxt, nonce=None):
        '''
        Decrypt the given message, if no nonce is given the nonce will be
        extracted from the message
        '''
        if nonce is None:
            nonce = ctxt[:libnacl.crypto_secretbox_NONCEBYTES]
            ctxt = ctxt[libnacl.crypto_secretbox_NONCEBYTES:]
        if len(nonce) != libnacl.crypto_secretbox_NONCEBYTES:
            raise ValueError('Invalid nonce')
        return libnacl.crypto_secretbox_open_easy(ctxt, nonce, self.sk)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

NAME = "libnacl"
DESC = "Python bindings for libsodium based on ctypes"

# Version info -- read without importing
_locals = {}
with open("libnacl/version.py") as fp:
    exec(fp.read(), None, _locals)
VERSION = _locals["version"]

setup(
    name=NAME,
    version=VERSION,
    description=DESC,
    author="Thomas S Hatch",
    author_email="thatch@saltstack.com",
    url="https://libnacl.readthedocs.org/",
    classifiers=[
        "Operating System :: OS Independent",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Security :: Cryptography",
    ],
    packages=["libnacl"],
)



# Import nacl libs
import libnacl
from hashlib import sha256, sha512

# Import python libs
import unittest


class TestHash(unittest.TestCase):
    """
    Test sign functions
    """
    def test_hash(self):
        msg1 = b'Are you suggesting coconuts migrate?'
        msg2 = b'Not at all, they could be carried.'
        chash1 = libnacl.crypto_hash(msg1)
        chash2 = libnacl.crypto_hash(msg2)
        self.assertNotEqual(msg1, chash1)
        self.assertNotEqual(msg2, chash2)
        self.assertNotEqual(chash2, chash1)

        ref256 = sha256(msg1)
        self.assertEqual(ref256.digest_size, libnacl.crypto_hash_sha256_BYTES)
        self.assertEqual(ref256.digest(), libnacl.crypto_hash_sha256(msg1))

        ref512 = sha512(msg1)
        self.assertEqual(ref512.digest_size, libnacl.crypto_hash_sha512_BYTES)
        self.assertEqual(ref512.digest(), libnacl.crypto_hash_sha512(msg1))




# -*- coding: utf-8 -*-

import struct
import sys
import time

# Import nacl libs
import libnacl
import libnacl.encode
import libnacl.secret
import libnacl.public
import libnacl.sign
import libnacl.dual


def load_key(path_or_file, serial='json'):
    '''
    Read in a key from a file and return the applicable key object based on
    the contents of the file
    '''
    if hasattr(path_or_file, 'read'):
        stream = path_or_file
    else:
        if serial == 'json':
            stream = open(path_or_file, 'r')
        else:
            stream = open(path_or_file, 'rb')

    try:
        if serial == 'msgpack':
            import msgpack
            key_data = msgpack.load(stream)
        elif serial == 'json':
            import json
            if sys.version_info[0] >= 3:
                key_data = json.loads(stream.read())
            else:
                key_data = json.loads(stream.read(), encoding='UTF-8')
    finally:
        if stream != path_or_file:
            stream.close()

    if 'priv' in key_data and 'sign' in key_data and 'pub' in key_data:
        return libnacl.dual.DualSecret(
                libnacl.encode.hex_decode(key_data['priv']),
                libnacl.encode.hex_decode(key_data['sign']))
    elif 'priv' in key_data and 'pub' in key_data:
        return libnacl.public.SecretKey(
                libnacl.encode.hex_decode(key_data['priv']))
    elif 'sign' in key_data:
        return libnacl.sign.Signer(
                libnacl.encode.hex_decode(key_data['sign']))
    elif 'pub' in key_data:
        return libnacl.public.PublicKey(
                libnacl.encode.hex_decode(key_data['pub']))
    elif 'verify' in key_data:
        return libnacl.sign.Verifier(key_data['verify'])
    elif 'priv' in key_data:
        return libnacl.secret.SecretBox(
                libnacl.encode.hex_decode(key_data['priv']))
    raise ValueError('Found no key data')


def salsa_key():
    '''
    Generates a salsa2020 key
    '''
    return libnacl.randombytes(libnacl.crypto_secretbox_KEYBYTES)


def aead_key():
    '''
    Generates an AEAD key (both implementations use the same size)
    '''
    return libnacl.randombytes(libnacl.crypto_aead_aes256gcm_KEYBYTES)


def rand_aead_nonce():
    '''
    Generates and returns a random bytestring of the size defined in libsodium
    as crypto_aead_aes256gcm_NPUBBYTES and crypto_aead_chacha20poly1305_ietf_NPUBBYTES
    '''
    return libnacl.randombytes(libnacl.crypto_aead_aes256gcm_NPUBBYTES)


def rand_nonce():
    '''
    Generates and returns a random bytestring of the size defined in libsodium
    as crypto_box_NONCEBYTES
    '''
    return libnacl.randombytes(libnacl.crypto_box_NONCEBYTES)


def time_nonce():
    '''
    Generates and returns a nonce as in rand_nonce() but using a timestamp for the first 8 bytes.

    This function now exists mostly for backwards compatibility, as rand_nonce() is usually preferred.
    '''
    nonce = rand_nonce()
    return (struct.pack('=d', time.time()) + nonce)[:len(nonce)]
